/*
 * arduino+M5310A实时控制
 * 2020-07-30
 * QQ交流群：824273231
 * 微信：19092550573
 * 官网https://bemfa.com 
 */
#include <SoftwareSerial.h>
SoftwareSerial mySerial(13, 12); // RX, TX  


/***********************需要修改的地方************************/
String uid ="4d9ec352e0376f2110a0c601a2857225";    //用户私钥，巴法云控制台获取
String myTopic ="light002";                      //用户主题，巴法云控制台创建
const int LED_Pin = 2;       //需要控制的LED引脚值
/***********************************************************/


int socketid = 1;//socket端口编号
#define KEEPALIVEATIME 30*1000     //心跳间隔，默认30秒发一次心跳
unsigned long preHeartTick = 0;//心跳时间
#define TIMEOUT 3000 //接收esp8266反馈的超时时间

int errorFlag = 0; //记录错误次数，错误次数过多，执行重启
/*
 * 打开灯泡函数
 */
void turnOnLed(){
  Serial.println("Turn ON");  //打印调试信息
  digitalWrite(LED_Pin,LOW); //改变引脚状态
}


/*
 * 关闭灯泡函数
 */
void turnOffLed(){
  Serial.println("Turn OFF");//打印调试信息
  digitalWrite(LED_Pin,HIGH);//改变引脚状态
}


   
/*
 * 重启函数，执行重启
 */
void(* resetFunc) (void) = 0;//重启函数


/*
 * 检查收到的信息 
 * 字符串匹配，匹配到开灯指令，进行开灯，匹配到关灯指令，进行关灯
 * 匹配到错误信息，进行重启启动(一般为网络情况的故障)
 */
void check_msg(String myMsg){
  if((myMsg.indexOf("&msg=on") >= 0)) {   //如果检测到开灯指令
    turnOnLed();  //执行开灯函数
  }else if((myMsg.indexOf("&msg=off") >= 0)) { //如果检测到关灯指令
    turnOffLed(); //执行关灯函数
  }else if(myMsg.indexOf("SOCLI") >= 0){  //检测到断开服务器连接，重新连接
       InitNBIOT();//创建TCP连接，并且订阅巴法云
    }
}

/*
 * 
 * 系统初始化
 * 初始化串口，并初始化WIFI
 */
void setup()
{
 delay(1000);
 Serial.begin(9600);  //arduino波特率，仅用于查看调试信息，连接串口调试助手可以查看
 mySerial.begin(9600);  //设置软串口波特率，建议9600，如果波特率过大，接收的指令存在乱码的情况（115200实测会乱码）
 mySerial.println("AT+NRB");//M5310重启指令，重启NB模块，适用于海思系列模块
 delay(8000);  //等待模块联网
 Serial.println("beginning...");  //打印调试信息
 while(mySerial.read()>= 0){}//清空软串口数据，等待下次接收
 while(SendCommand("AT","OK",1500)){//发送AT指令，检查模块是否正常启动
    delay(1000);//如果错误返回，等待1秒重新发送AT
    errorFlag++;//记录错误次数
    if(errorFlag>=5){//如果连续错误5次，执行重启
      mySerial.println("AT+NRB");//M5310重启指令，重启NB模块，适用于海思系列模块
      delay(200);
      resetFunc(); //重启函数，执行重启arduino
      }
    }
 errorFlag = 0;//错误次数清零
 while(mySerial.read()>= 0){}//清空软串口数据，等待下次接收
  while(SendCommand("AT+CEREG?","1",1500)){//检查是否驻网成功，即检查是否正确连接NB网络
    delay(2000);//如果错误返回，等待1秒重新发送AT
    errorFlag++;//记录错误次数
    if(errorFlag>=5){//如果连续错误5次，执行重启
      mySerial.println("AT+NRB");//M5310重启指令，重启NB模块，适用于海思系列模块
      delay(200);
      resetFunc(); //重启函数，执行重启
      }
    }
 errorFlag = 0;//错误次数清零
 delay(1000);
 InitNBIOT();//创建TCP连接，并且订阅巴法云
}
 
void loop(){

 String IncomingString="";  //用于接收串口发来的数据
 boolean StringReady = false;  //接收到串口数据的标志
 while (mySerial.available()){//如果接收到esp8266的数据
   IncomingString=mySerial.readString(); //获取esp8266反馈的数据，及esp8266收到远程服务器发来的数据
   StringReady= true;//接收到数据的标志
  }
 
  if (StringReady){ //如果有数据发来，检查接收到的数据
    Serial.println("Received String: " + IncomingString);  //串口打印显示收到的数据
    check_msg(IncomingString);  //调用检查数据函数，进行检查数据
  }

  if(millis() - preHeartTick >= KEEPALIVEATIME){//定时函数，用于保持心跳，30秒检测一次（现在时间减去上次时间是否大于或等于30s）
      preHeartTick = millis(); //获取现在时间，用于下次计算
          while(SendCommand("AT+NSOSD="+String(socketid)+",1,\"cmd=0&msg=keep\\r\\n\",,4","OK",1500)){//发送心跳
              delay(2000);//如果发送失败，延迟2秒后继续发送
              errorFlag++;//累加错误次数
              if(errorFlag>=2){//如果错误达到2次
                  errorFlag = 0;//清空错误次数
                  mySerial.println("AT+NSOCL="+String(socketid));//创建TCP连接
                  socketid++;
                  if(socketid>=7){
                      socketid=1;
                    }
                  InitNBIOT();//重新连接网络
              } 
           }
          errorFlag = 0;
  }
}

/*
 * 发送AT指令函数
 * ****************************
 * cmd为需要发送的指令
 * ack是期待收到的结果
 * ****************************
 */
boolean SendCommand(String cmd, String ack,int timeout){
  mySerial.println(cmd); // 向软串口发送指令
  Serial.println(cmd); // 串口调试助手打印指令信息
  if (!echoFind(ack,timeout)) // 如果超时或者错误响应
  {
    return true; // 返回真，说明指令发送失败
    }else{
      return false;//指令发送成功
      }
    
}
/*
 * 检查收到的字符串
 * 配合SendCommand函数使用
 */
boolean echoFind(String keyword,int TimeOut){
 long deadline = millis() + TimeOut;//设置超时时间
 String get_msg="";//用于接收软串口数据
 while(millis() < deadline){//设置检测软串口时间
    if (mySerial.available()){//如果软串口有数据
         get_msg = mySerial.readString();//读取软串口数据
    }
  }
  if(get_msg !=""){//如果接收到数据
       Serial.println(get_msg);//串口打印收到的数据
       check_msg(get_msg);  //检查数据
       int keyword_index = get_msg.indexOf(keyword);//获取关键字所在字符串位置
       if(keyword_index >=0){//如果接收到的字符串有期待接收到的关键字
          return true;  //返回真
        }
  }
 return false; // 超时或无检测数据
}

/*
 * 初始化M5310
 * 创建TCP连接，并且订阅巴法云
 */
void InitNBIOT(void){
 delay(500);
 while(mySerial.read()>= 0){}//清空软串口数据，等待下次接收
 mySerial.println("AT+NSOCR=\"STREAM\",6,0,2");//创建TCP连接
 delay(2000);
 while(SendCommand("AT+NSOCO="+String(socketid)+",bemfa.com,8344","CONNECT",3500)){//连接巴法云服务器
    delay(2000);//如果错误返回，等待1秒重新发送AT
    errorFlag++;//记录错误次数
    if(errorFlag>=3){//如果连续错误5次，执行重启
      mySerial.println("AT+NRB");//M5310重启指令，重启NB模块，适用于海思系列模块
      delay(200);
      resetFunc(); //重启函数，执行重启arduino
    }
 }
 errorFlag = 0;//错误次数清零
 while(mySerial.read()>= 0){}//清空软串口数据，等待下次接收
 delay(1000);
 mySerial.println("AT+NSOCFG="+String(socketid)+",0,0");//设置字符串传输模式，1代表socket1,第一个0代表字符串模式接收，第二个0代表转义字符模式发送
 delay(300);
 SendCommand("AT+NSOSD="+String(socketid)+",1,\"cmd=1&uid="+uid+"&topic="+myTopic+"\\r\\n\",,4","OK",1500);  // 发送订阅指令，如需订阅多个多个主题，可延迟一秒后继续发送订阅
}
